# -*- coding: utf8 -*-

from multiprocessing import cpu_count
from multiprocessing.pool import ThreadPool

from noseapp.suite.bases.simple import RunSuite
from noseapp.suite.bases.simple import BaseSuite


class ThreadingRunSuite(RunSuite):

    def perform(self):
        size = self._suite.config.options.async_tests

        if size < 0:
            size = cpu_count() / 2

        pool = ThreadPool(int(round(size)) or 2)

        for test in self._suite.tests:
            if self._result.shouldStop:
                break

            pool.apply_async(self.run_test, args=(test,))

        pool.close()
        pool.join()


class ThreadSuite(BaseSuite):
    """
    Run tests with multiprocessing.pool.ThreadPool
    """

    run_suite_class = ThreadingRunSuite



# -*- coding: utf8 -*-

from unittest import TestCase


class TestClearExtensions(TestCase):
    """
    Set extension, check get extension, call clear, check raise
    """

    def runTest(self):
        from noseapp.core import extensions

        extensions.set('test', dict(test='test'))
        self.assertEqual(extensions.get('test').get('test'), 'test')

        extensions.clear()
        self.assertRaises(extensions.ExtensionNotFound, extensions.get, 'test')


class TestClassFactory(TestCase):
    """
    Test ClassFactory usage
    """

    def runTest(self):
        from noseapp.runner.base import BaseTestRunner
        from noseapp.core.constants import RunStrategy
        from noseapp.suite.bases.simple import BaseSuite
        from noseapp.suite.bases.threading_suite import ThreadSuite
        from noseapp.runner.threading_runner import ThreadingTestRunner
        from noseapp.runner.multiprocessing_runner import MultiprocessingTestRunner

        from noseapp.core.factory import ClassFactory

        class MockOptions(object):
            def __init__(self, **kw):
                self.run_strategy = kw.pop('run_strategy', RunStrategy.SIMPLE)
                self.async_suites = kw.pop('async_suites', 0)
                self.async_tests = kw.pop('async_tests', 0)

        factory = ClassFactory(MockOptions())

        self.assertEqual(factory.suite_class, BaseSuite)
        self.assertEqual(factory.runner_class, BaseTestRunner)

        factory = ClassFactory(MockOptions(async_suites=2))

        self.assertEqual(factory.suite_class, BaseSuite)
        self.assertEqual(factory.runner_class, MultiprocessingTestRunner)

        factory = ClassFactory(MockOptions(async_suites=2, async_tests=2))

        self.assertEqual(factory.suite_class, ThreadSuite)
        self.assertEqual(factory.runner_class, MultiprocessingTestRunner)

        factory = ClassFactory(
            MockOptions(run_strategy=RunStrategy.MULTIPROCESSING, async_suites=2),
        )

        self.assertEqual(factory.suite_class, BaseSuite)
        self.assertEqual(factory.runner_class, MultiprocessingTestRunner)

        factory = ClassFactory(
            MockOptions(run_strategy=RunStrategy.MULTIPROCESSING, async_suites=2, async_tests=2),
        )

        self.assertEqual(factory.suite_class, ThreadSuite)
        self.assertEqual(factory.runner_class, MultiprocessingTestRunner)

        factory = ClassFactory(
            MockOptions(run_strategy=RunStrategy.THREADING, async_suites=2),
        )

        self.assertEqual(factory.suite_class, BaseSuite)
        self.assertEqual(factory.runner_class, ThreadingTestRunner)

        factory = ClassFactory(
            MockOptions(run_strategy=RunStrategy.THREADING, async_suites=2, async_tests=2),
        )

        self.assertEqual(factory.suite_class, ThreadSuite)
        self.assertEqual(factory.runner_class, ThreadingTestRunner)

        try:  # if gevent installed only
            from noseapp.suite.bases.gevent_suite import GeventSuite
            from noseapp.runner.gevent_runner import GeventTestRunner

            factory = ClassFactory(
                MockOptions(run_strategy=RunStrategy.GEVENT, async_suites=2),
            )

            self.assertEqual(factory.suite_class, BaseSuite)
            self.assertEqual(factory.runner_class, GeventTestRunner)

            factory = ClassFactory(
                MockOptions(run_strategy=RunStrategy.GEVENT, async_suites=2, async_tests=2),
            )

            self.assertEqual(factory.suite_class, GeventSuite)
            self.assertEqual(factory.runner_class, GeventTestRunner)
        except ImportError:
            pass



# -*- coding: utf8 -*-

import sys

from nose.core import TestProgram as BaseTestProgram

from noseapp.app import config
from noseapp.core import extensions


class ProgramData(object):
    """
    Merging data of test program and application
    """

    def __init__(self, program, app, argv=None):
        # Push options from parser to application
        app.options = app.config_class(
            config.load(program.config.options),
        )

        # Initialize application
        app.initialize()

        self.__argv = argv or sys.argv
        self.__class_factory = app.class_factory(program.config.options)

        self.__app = app
        self.__program = program

    @property
    def argv(self):
        return self.__argv

    @property
    def suites(self):
        return self.__app.suites

    @property
    def test_loader(self):
        return self.__program.testLoader

    @property
    def config(self):
        return self.__program.config

    @property
    def suite_class(self):
        return self.__class_factory.suite_class

    @property
    def runner_class(self):
        return self.__class_factory.runner_class

    def before_run_callback(self):
        self.__app.before()

    def after_run_callback(self):
        self.__app.after()

    def build_suites(self):
        """
        Collect and build suites
        """
        collector = self.__app.collector_class(self)
        suites = collector.make_result()
        extensions.clear()
        return suites


class TestProgram(BaseTestProgram):

    def __init__(self, *args, **kwargs):
        app = kwargs.pop('app')

        super(TestProgram, self).__init__(*args, **kwargs)

        self.data = ProgramData(
            self, app,
            argv=kwargs.get('argv', None),
        )

    # disarm base class
    def createTests(self):
        pass

    def runTests(self):
        pass

    def perform(self):
        """
        Perform test program
        """
        self.testRunner = self.data.runner_class(
            stream=self.config.stream,
            verbosity=self.config.verbosity,
            config=self.config,
        )

        plug_runner = self.config.plugins.prepareTestRunner(self.testRunner)

        if plug_runner is not None:
            self.testRunner = plug_runner

        suites = self.data.build_suites()

        if not suites:
            raise RuntimeError('No suites for running')

        self.data.before_run_callback()

        result = self.testRunner.run(suites)
        self.success = result.wasSuccessful()

        self.data.after_run_callback()

        if self.exit:
            sys.exit(not self.success)

        return self.success




import os
import time
from optparse import OptionGroup

from noseapp.plugins.base import AppPlugin
from noseapp.core.constants import RunStrategy


class AppConfigurePlugin(AppPlugin):

    name = 'app-configure'

    def options(self, parser, env=os.environ):
        super(AppConfigurePlugin, self).options(parser, env=env)
        group = OptionGroup(parser, 'NoseApp base options')

        group.add_option(
            '--run-strategy',
            dest='run_strategy',
            default=RunStrategy.SIMPLE,
            type=str,
            help='Can be in ({})'.format(', '.join(RunStrategy.ALL)),
        )
        group.add_option(
            '--async-suites',
            dest='async_suites',
            default=0,
            type=int,
            help='Num of suites to async run.',
        )
        group.add_option(
            '--async-tests',
            dest='async_tests',
            default=0,
            type=int,
            help='Num of tests to async run. Sets limit inside suite.',
        )
        group.add_option(
            '--multiprocessing-timeout',
            dest='multiprocessing_timeout',
            default=1800,
            type=int,
            help='Process timeout. With multiprocessing strategy only.',
        )
        group.add_option(
            '--ls',
            dest='ls',
            action='store_true',
            default=False,
            help='Show suites tree.'
        )
        group.add_option(
            '--doc',
            dest='doc',
            action='store_true',
            default=False,
            help='Show docstring of test cases.'
        )
        group.add_option(
            '-t', '--test',
            dest='run_test',
            default='',
            type=str,
            help='Test case or suite name for running.'
        )
        group.add_option(
            '--random',
            dest='random',
            action='store_true',
            default=False,
            help='Random shuffle for cases'
        )
        group.add_option(
            '--random-seed',
            dest='random_seed',
            default=int(time.time()),
            type=int,
            help='Seed of random shuffle. Using with random option only.'
        )

        parser.add_option_group(group)

        try:  # if app instance does not pushed to plugin,
            # AttributeError will be passed.
            self.app.add_options(parser)
        except AttributeError:
            pass



# -*- coding: utf-8 -*-


class RunStrategy(object):
    """
    Strategy for run tests
    """

    SIMPLE = 'simple'
    GEVENT = 'gevent'
    THREADING = 'threading'
    MULTIPROCESSING = 'multiprocessing'

    ALL = (
        SIMPLE,
        GEVENT,
        THREADING,
        MULTIPROCESSING,
    )



# -*- coding: utf8 -*-

from multiprocessing import cpu_count

from gevent.pool import Pool

from noseapp.suite.bases.simple import RunSuite
from noseapp.suite.bases.simple import BaseSuite


class GeventRunSuite(RunSuite):

    def perform(self):
        size = self._suite.config.options.async_tests

        if size < 0:
            size = cpu_count() - 1

        pool = Pool(int(round(size)) or 2)

        for test in self._suite.tests:
            if self._result.shouldStop:
                break

            pool.spawn(self.run_test, test)

        pool.join()


class GeventSuite(BaseSuite):
    """
    Run tests with gevent pool
    """

    run_suite_class = GeventRunSuite



# -*- coding: utf8 -*-

from nose.suite import ContextSuite


class RunSuite(object):

    def __init__(self, suite, result, orig):
        self._orig = orig
        self._suite = suite
        self._result = result

    def run_test(self, test):
        for handler in self._suite.handlers:
            try:
                handler(test.test)
            except KeyboardInterrupt:
                raise
            except:
                self._suite.error_context = 'setup'
                self._orig.addError(self, self._suite._exc_info())
                return

        test(self._orig)

    def perform(self):
        for test in self._suite.tests:
            if self._result.shouldStop:
                break

            self.run_test(test)


class BaseSuite(ContextSuite):
    """
    Base suite
    """

    run_suite_class = RunSuite

    def __init__(self, *args, **kwargs):
        self._handlers = kwargs.pop('handlers', [])

        super(BaseSuite, self).__init__(*args, **kwargs)

    @property
    def tests(self):
        return self._tests

    @property
    def handlers(self):
        return self._handlers

    def run(self, result):
        if self.resultProxy:
            result, orig = self.resultProxy(result, self), result
        else:
            result, orig = result, result

        try:
            self.setUp()
        except KeyboardInterrupt:
            raise
        except:
            self.error_context = 'setup'
            result.addError(self, self._exc_info())
            return

        try:
            runner = self.run_suite_class(self, result, orig)
            runner.perform()
        finally:
            self.has_run = True

            try:
                self.tearDown()
            except KeyboardInterrupt:
                raise
            except:
                self.error_context = 'teardown'
                result.addError(self, self._exc_info())



# -*- coding: utf-8 -*-

from unittest import TestCase

from noseapp import manage


class TestRegisterCommand(TestCase):

    def runTest(self):
        fake_command = lambda: None

        manage.register_command('fake_command', fake_command)

        self.assertIn(fake_command, manage._COMMANDS.values())
